import unittest
import contextlib
import io

from numpy.testing import assert_array_equal
from animalai.raycastparser import RayCastParser, RayCastObjects

""" This file contains tests for the raycastparser class. Please keep it this way. """


class TestRayCastParser(unittest.TestCase):
    def test_GOODGOAL_IMMOVABLE(self):
        """
        Check if the parser correctly identifies GOODGOAL and IMMOVABLE objects
        and places them correctly in the parsed raycast array.
        """
        parser = RayCastParser(
            [RayCastObjects.GOODGOAL, RayCastObjects.IMMOVABLE], 5)
        # fmt: off
        test_raycast = [1, 1, 1, 1, 1, 1, 0, 0.1,
                        1, 1, 1, 1, 1, 1, 0, 0.2,
                        1, 1, 1, 1, 1, 1, 1, 0.3,
                        1, 1, 1, 1, 1, 1, 1, 0.4,
                        1, 1, 1, 1, 1, 1, 1, 0.5]
        parsed_raycast = parser.parse(test_raycast)
        assert_array_equal(parsed_raycast, [
            [0.3, 0.5, 0.1, 0.4, 0.2],
            [0.3, 0.5, 0.1, 0.4, 0.2]
        ])

        # Test prettyprint
        pp_out = io.StringIO()
        with contextlib.redirect_stdout(pp_out):
            parser.prettyPrint(test_raycast)

        self.assertEqual(pp_out.getvalue(),
            "GOODGOAL : [0.3 0.5 0.1 0.4 0.2]\n" +
            "IMMOVABLE : [0.3 0.5 0.1 0.4 0.2]\n",
        )
        # fmt: on

    def test_NO_OBJECTS(self):
        """
        Checks if the parser correctly identifies when no objects are detected.
        """
        parser = RayCastParser(
            [RayCastObjects.GOODGOAL, RayCastObjects.BADGOAL], 3)
        # fmt: off
        test_raycast = [0, 0, 0, 0, 0, 0, 0, 0.1,
                        0, 0, 0, 0, 0, 0, 0, 0.2,
                        0, 0, 0, 0, 0, 0, 0, 0.3]
        parsed_raycast = parser.parse(test_raycast)
        assert_array_equal(parsed_raycast, [
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0]
        ])
        # Test prettyprint
        pp_out = io.StringIO()
        with contextlib.redirect_stdout(pp_out):
            parser.prettyPrint(test_raycast)

        self.assertEqual(pp_out.getvalue(),
            "GOODGOAL : [0. 0. 0.]\n" + 
            "BADGOAL : [0. 0. 0.]\n")
        # fmt: on

    def test_MIXED_OBJECTS(self):
        """
        Checks if the parser correctly identifies some objects while ignoring others.
        """
        parser = RayCastParser(
            [
                RayCastObjects.ARENA,
                RayCastObjects.MOVABLE,
                RayCastObjects.GOODGOALMULTI,
            ],
            7,
        )
        # fmt: off
        test_raycast = [1, 0, 0, 0, 0, 0, 0, 0.1,
                        0, 1, 0, 0, 0, 0, 0, 0.2,
                        0, 0, 1, 0, 0, 0, 1, 0.3,
                        0, 0, 0, 1, 0, 0, 1, 0.4,
                        0, 0, 0, 0, 1, 0, 1, 0.5,
                        0, 0, 0, 0, 0, 1, 1, 0.6,
                        0, 0, 0, 0, 0, 0, 0, 0]
        parsed_raycast = parser.parse(test_raycast)
        assert_array_equal(parsed_raycast, [
            [0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0],
            [0.3, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            [0.0, 0.5, 0.0, 0.0, 0.0, 0.0, 0.0 ]
            ])
        # Test prettyprint
        pp_out = io.StringIO()
        with contextlib.redirect_stdout(pp_out):
            parser.prettyPrint(test_raycast)

        self.assertEqual(pp_out.getvalue(),
            "ARENA : [0.  0.  0.  0.1 0.  0.  0. ]\n" + 
            "MOVABLE : [0.3 0.  0.  0.  0.  0.  0. ]\n" + 
            "GOODGOALMULTI : [0.  0.5 0.  0.  0.  0.  0. ]\n")
        # fmt: on

    def test_MIX_PILLARBUTTON(self):
        """
        Check if the parser correctly identifies some objects including PILLARBUTTON while ignoring others.
        """
        parser = RayCastParser(
            [RayCastObjects.ARENA, RayCastObjects.PILLARBUTTON, RayCastObjects.MOVABLE],
            7,
        )
        # fmt: off
        test_raycast = [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0.1,
                        0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0.2,
                        0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0.3,
                        0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0.4,
                        0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0.5,
                        0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0.6,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0]
        parsed_raycast = parser.parse(test_raycast)
        assert_array_equal(parsed_raycast, [
            [0.0,  0.0,  0.0,  0.1, 0.0,  0.0,  0.0],
            [0.3, 0.5, 0.0, 0.1, 0.6, 0.4, 0.2],
            [0.3, 0.0,  0.0,  0.0,  0.0,  0.0,  0.0]
            ])
        # Test prettyprint
        pp_out = io.StringIO()
        with contextlib.redirect_stdout(pp_out):
            parser.prettyPrint(test_raycast)

        self.assertEqual(pp_out.getvalue(),
            "ARENA : [0.  0.  0.  0.1 0.  0.  0. ]\n" +
            "PILLARBUTTON : [0.3 0.5 0.  0.1 0.6 0.4 0.2]\n" + 
            "MOVABLE : [0.3 0.  0.  0.  0.  0.  0. ]\n")
        # fmt: on



import numpy as np
from collections import deque
from typing import Deque
from animalai.actions import AAIActions, AAIAction


class RandomActionAgent:
    """
    Implements a random action agent. Chooses one of 9 actions, and then repeats it for a number of steps before picking a new action.
    Parameters for distributions for choosing step lengths, biases towards certain actions, and correlations with previous biases.
    
    Parameters
    ----------
    max_step_length : int
        The maximum number of steps the agent should take of one action before picking another. Ignored unless `step_length_distribution` = None.
    step_length_distribution : np.random.* lambda function 
        Distribution from which to sample number of steps before picking a new action. Absolute value is taken from these distributions, rounded to the nearest integer. 
        Passing `lambda: np.random.standard_cauchy()` simulates a Levy walker, drawing step lengths from the cauchy distribution. `lambda: np.random.normal(5, 1)` simulates a Rayleigh walker, drawing step lengths from the normal distribution. 
        Default is None (fixed length walker)
    action_biases : list
        A list of biases for each of the action, ordered as [NOOP,LEFT,RIGHT,FORWARDS,BACKWARDS,FORWARDSLEFT,FORWARDSRIGHT,BACKWARDSLEFT,BACKWARDSRIGHT]. 
        These values are softmaxed, so relative magnitude is important. Default is [1,1,1,1,1,1,1,1,1]
    prev_step_bias : float
        Probability that the next action to be selected is the previous action. Must be between 0 and 1 inclusive. Default is 0.
    remove_prev_step : bool
        Exclude the previous step when selecting next action, to ensure diversity of actions. Default is False. 
    """

    def __init__(self, max_step_length=10, step_length_distribution=None, action_biases=None, prev_step_bias=0, remove_prev_step=False):
        self.max_step_length = max_step_length
        self.step_length_distribution = step_length_distribution if step_length_distribution is not None else lambda: max_step_length
        self.action_biases = action_biases if action_biases is not None else [1] * 9
        self.prev_step_bias = prev_step_bias
        self.remove_prev_step = remove_prev_step
        self.actions = [AAIActions().NOOP, 
                        AAIActions().LEFT, 
                        AAIActions().RIGHT, 
                        AAIActions().FORWARDS, 
                        AAIActions().BACKWARDS, 
                        AAIActions().FORWARDSLEFT, 
                        AAIActions().FORWARDSRIGHT, 
                        AAIActions().BACKWARDSLEFT, 
                        AAIActions().BACKWARDSRIGHT,
                        ]

    def get_num_steps(self, prev_step: AAIAction) -> Deque[AAIAction]:
        num_steps = abs(int(self.step_length_distribution()))

        step_list = deque([prev_step] * num_steps)
        return step_list

    def get_new_action(self, prev_step: AAIAction) -> AAIAction:
        """
        Provide a vector of 9 real values, one for each action, which is then softmaxed to provide the probability of selecting that action. Relative differences between the values is what is important. 

        Provide an initial probability of selecting the previous step again. If that action is not selected, then the next step is picked according to the softmaxed action biases. The previous action can be removed
        from the softmaxed biases (by continually sampling until an action is picked that is not the previous action), by changing `remove_prev_step` to `True`.
        """
        assert len(self.action_biases) == 9, "You must provide biases for all nine (9) actions. A uniform distribution is [1,1,1,1,1,1,1,1,1]"
        assert 0 <= self.prev_step_bias <= 1, "The bias towards the previous action must be a scalar value between 0 and 1."

        action_is_prev_step = np.random.choice(a=[False, True], size=1, p=[(1 - self.prev_step_bias), self.prev_step_bias])

        if action_is_prev_step:
            return prev_step
        else:
            action_biases_softmax = (np.exp(self.action_biases) / np.sum(np.exp(self.action_biases))) 
            if self.remove_prev_step:
                action = prev_step
                while action == prev_step:
                    action = np.random.choice(a=self.actions, size=1, p=action_biases_softmax)
            else:
                action = np.random.choice(a=self.actions, size=1, p=action_biases_softmax)

            return action[0]


import unittest
from unittest.mock import patch
from animalai.environment import AnimalAIEnvironment
from mlagents_envs.side_channel.raw_bytes_channel import RawBytesChannel
from mlagents_envs.side_channel.engine_configuration_channel import (
    EngineConfigurationChannel,
)

""" This file contains tests for the environment class. Please keep it this way. """


class TestEnvironmentInitialization(unittest.TestCase):
    @patch("animalai.environment.UnityEnvironment")
    def test_initialization_with_default_parameters(self, mock_env):
        """Test initialization with default parameters."""
        env = AnimalAIEnvironment()
        mock_env.assert_called_once()
        self.assertIsNotNone(env)

    @patch("animalai.environment.UnityEnvironment")
    def test_initialization_with_custom_parameters(self, mock_env):
        """Test initialization with custom parameters."""
        env = AnimalAIEnvironment(
            file_name="custom_file.yml", worker_id=2
        )  # For custom file path, use full path.
        mock_env.assert_called_once()
        self.assertIsNotNone(env)


class TestSideChannelsConfiguration(unittest.TestCase):
    def test_side_channels_configuration(self):
        """Test if side channels are correctly configured."""
        try:
            env = AnimalAIEnvironment()

            # Check for specific types of side channels
            has_engine_config_channel = any(
                isinstance(channel, EngineConfigurationChannel)
                for channel in env.side_channels
            )
            self.assertTrue(
                has_engine_config_channel, "EngineConfigurationChannel not found"
            )

        except Exception as e:
            self.fail(f"Side channels configuration test failed: {e}")


if __name__ == "__main__":
    unittest.main()



import numpy as np
from mlagents_envs.base_env import ActionTuple


## Python Class to help access the discrete action space of AAI environment.
class AAIActions:
    def __init__(self, no_agents=1):
        if not isinstance(no_agents, int) or no_agents < 1:
            raise ValueError("no_agents must be a positive integer.")

        self.NOOP = AAIAction(
            "noop",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[0, 0]], dtype=np.int32),
            ),
        )
        self.LEFT = AAIAction(
            "left",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[0, 2]], dtype=np.int32),
            ),
        )
        self.RIGHT = AAIAction(
            "right",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[0, 1]], dtype=np.int32),
            ),
        )
        self.FORWARDS = AAIAction(
            "forwards",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[1, 0]], dtype=np.int32),
            ),
        )
        self.FORWARDSLEFT = AAIAction(
            "forwards&left",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[1, 2]], dtype=np.int32),
            ),
        )
        self.FORWARDSRIGHT = AAIAction(
            "forwards&right",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[1, 1]], dtype=np.int32),
            ),
        )
        self.BACKWARDS = AAIAction(
            "backwards",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[2, 0]], dtype=np.int32),
            ),
        )
        self.BACKWARDSLEFT = AAIAction(
            "backwards&left",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[2, 2]], dtype=np.int32),
            ),
        )
        self.BACKWARDSRIGHT = AAIAction(
            "backwards&right",
            ActionTuple(
                continuous=np.zeros((no_agents, 0)),
                discrete=np.array([[2, 1]], dtype=np.int32),
            ),
        )
        self.allActions: list = [
            self.NOOP,
            self.LEFT,
            self.RIGHT,
            self.FORWARDS,
            self.FORWARDSLEFT,
            self.FORWARDSRIGHT,
            self.BACKWARDS,
            self.BACKWARDSLEFT,
            self.BACKWARDSRIGHT,
        ]

    def random(self):
        return np.random.choice(self.allActions)


class AAIAction:
    """Actions have a name and an associated ActionTuple"""

    def __init__(self, name, action_tuple):
        if not isinstance(name, str):
            raise TypeError("name must be a string.")
        if not isinstance(action_tuple, ActionTuple):
            raise TypeError("action_tuple must be an instance of ActionTuple.")
        self.name = name
        self.action_tuple = action_tuple

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"AAIAction(name={self.name}, action_tuple={self.action_tuple})"



from pathlib import Path


def find_executable(base: Path) -> Path:
    """
    Look for the latest version of the AAI.

    We look in 'BASE/env/' for files matching {AAI,AnimalAI}.{x86_64,exe,app}, e.g. AAI.x86_64.
    Afterward, we look in the folders matching 'BASE/env*/',taking the one that is lexicographically last, e.g. 'BASE/env3.1.3/'.
    """
    error_msg = (
        "Could not automatically find any AAI environment binaries.\n\n"
        "We look in $BASE$/env/ for files matching {AAI,AnimalAI}.{x86_64,exe,app}, e.g. AAI.x86_64. "
        "Afterward, we look in the folders matching '$BASE$/env*/', "
        " taking the one that is lexicographically last, e.g. '$BASE$/env3.1.3/'.\n\n"
        "You can also specify the path exactly with the --env argument."
    ).replace("$BASE$", str(base))

    # Select folder
    env_folders = sorted(base.glob("env*"))
    if (base / "env/").exists():
        env_folders.append(base / "env/")
    if len(env_folders) == 0:
        reason = f"Could not find any folders matching {str(base)}/env*/"
        raise FileNotFoundError(f"{error_msg}\n\nReason: {reason}")
    env_folder = env_folders[-1]

    # Look for binary in selected folder
    # We brace expand manually because glob does not support it.
    binaries = [
        bin
        for bin_name in ["AAI", "AnimalAI"]
        for ext in ["x86_64", "exe", "app"]
        for bin in env_folder.glob(f"{bin_name}.{ext}")
    ]
    if len(binaries) == 0:
        reason = f"Could not find any AAI binaries in {env_folder}."
        raise FileNotFoundError(f"{error_msg}\n\nReason: {reason}")

    return binaries[0]



import enum
import numpy as np

"""
The script is designed to parse raycast observations from the Animal-AI environment. 
It includes a class to interpret these observations and output a simplified version 
of the data that only contains relevant objects. 
"""


class RayCastObjects(enum.Enum):
    """
    Enumeration of possible objects detected by the raycast.
    The values should correspond with how they are defined in the AnimalAI Unity environment.
    """

    ARENA = 0
    IMMOVABLE = 1
    MOVABLE = 2
    GOODGOAL = 3
    GOODGOALMULTI = 4
    BADGOAL = 5
    GOALSPAWNER = 6
    DEATHZONE = 7
    HOTZONE = 8
    RAMP = 9
    PILLARBUTTON = 10
    SIGNPOSTER = 11
    DECOYGOAL = 12
    DECOYGOALBOUNCE = 13
    HOLLOWBOX = 14


class RayCastParser:
    """
    The RayCastParser class is responsible for parsing raycast observations
    from the AnimalAI environment and returning a simplified version that only
    contains relevant objects.
    """

    def __init__(self, listOfObjects, numberOfRays):
        """
        Initialize the RayCastParser.

        Parameters:
        - listOfObjects: List of objects (from RayCastObjects enum) that the parser should look for.
        - numberOfRays: The number of rays in the raycast from AnimalAI environment.
        """
        # Validate inputs
        if not all(isinstance(item, RayCastObjects) for item in listOfObjects):
            raise ValueError(
                "listOfObjects must contain only elements of RayCastObjects enum."
            )
        if not isinstance(numberOfRays, int) or numberOfRays <= 0:
            raise ValueError("numberOfRays must be a positive integer.")

        self.numberOfRays = numberOfRays
        self.listOfObjects = listOfObjects
        self.listofObjectVals = [x.value for x in listOfObjects]
        self.numberOfObjects = len(listOfObjects)

    def parse(self, raycast) -> np.ndarray:
        """
        Parse the raw raycast data and return a simplified array.

        Parameters:
        - raycast: The raw raycast array from the AnimalAI environment.

        Returns:
        - np.ndarray: The parsed raycast, simplified to only include objects in listOfObjects.
        """
        # Validate and preprocess raycast input
        if not isinstance(raycast, (list, np.ndarray, dict)):
            raise TypeError(
                "raycast must be a list, numpy array, or dictionary.")

        if isinstance(raycast, dict):
            raycast = raycast.get("rays", [])
        if not isinstance(raycast, (list, np.ndarray)) or len(raycast) == 0:
            raise ValueError("Invalid raycast length or num of rays.")

        self.numberDetectableObjects = int(len(raycast) / self.numberOfRays) - 2
        parsedRaycast = np.zeros((len(self.listOfObjects), self.numberOfRays))
        for i in range(self.numberOfRays):
            for j in range(self.numberDetectableObjects):
                idx = i * (self.numberDetectableObjects + 2) + j
                distance_idx = (
                    i * (self.numberDetectableObjects + 2)
                    + self.numberDetectableObjects
                    + 1
                )
                if idx < len(raycast) and j in self.listofObjectVals:
                    if raycast[idx] == 1 and distance_idx < len(raycast):
                        parsedRaycast[self.listofObjectVals.index(j)][i] = raycast[
                            distance_idx
                        ]

        parsedRaycast = np.reshape(
            parsedRaycast, (len(self.listOfObjects), self.numberOfRays)
        )
        reordered = np.zeros_like(parsedRaycast)
        for i in range(parsedRaycast.shape[0]):
            reordered[i] = self.reorderRow(parsedRaycast[i])
        return reordered

    def reorderRow(self, row):
        """
        Reorder the elements in a row so that they read from left to right,
        as opposed to from the middle outwards.

        Parameters:
        - row: A 1D numpy array representing a single row of parsed raycast data.

        Returns:
        - np.ndarray: The reordered row.
        """
        newRow = np.zeros_like(row)
        midIndex = int((self.numberOfRays - 1) / 2)
        newRow[midIndex] = row[0]
        for i in range(midIndex):
            newRow[i + 1 + midIndex] = row[self.numberOfRays - 2 * (i + 1)]
            newRow[i] = row[2 * (i + 1)]
        return newRow

    def prettyPrint(self, raycast) -> None:
        """
        Pretty-prints the parsed raycast data, making it easier to read and understand.

        Parameters:
        - raycast: The raw raycast array from the AnimalAI environment.

        Prints the parsed and simplified raycast in a human-readable format.
        """
        # Validate and preprocess raycast input (same as in parse())
        try:
            parsedRaycast = self.parse(raycast)
            for i in range(parsedRaycast.shape[0]):
                print(self.listOfObjects[i].name, ":", parsedRaycast[i])
        except Exception as e:
            print(f"Error in parsing and printing raycast data: {e}")



from animalai.actions import AAIActions, AAIAction
from animalai.raycastparser import RayCastParser
from animalai.raycastparser import RayCastObjects

class Braitenberg():
    """Implements a simple Braitenberg vehicle agent that heads towards food
    Can change the number of rays but only responds to GOODGOALs, GOODGOALMULTI and BADGOAL"""
    def __init__(self, no_rays):
        self.no_rays = no_rays
        assert(self.no_rays % 2 == 1), "Only supports odd number of rays (but environment should only allow odd number"
        self.listOfObjects = [RayCastObjects.GOODGOAL, RayCastObjects.GOODGOALMULTI, RayCastObjects.BADGOAL, RayCastObjects.IMMOVABLE, RayCastObjects.MOVABLE]
        self.raycast_parser = RayCastParser(self.listOfObjects, self.no_rays)
        self.actions = AAIActions()
        self.prev_action = self.actions.NOOP

    def prettyPrint(self, obs) -> None:
        """Prints the parsed observation"""
        return self.raycast_parser.prettyPrint(obs)
    
    def get_action(self, obs) -> AAIAction:
        """Returns the action to take given the current parsed raycast observation"""
        obs = self.raycast_parser.parse(obs)
        newAction = self.actions.NOOP
        if self.ahead(obs, RayCastObjects.GOODGOALMULTI):
            newAction = self.actions.FORWARDS
        elif self.left(obs, RayCastObjects.GOODGOALMULTI):
            newAction = self.actions.LEFT
        elif self.right(obs, RayCastObjects.GOODGOALMULTI):
            newAction = self.actions.RIGHT
        elif self.ahead(obs, RayCastObjects.GOODGOAL):
            newAction = self.actions.FORWARDS
        elif self.left(obs, RayCastObjects.GOODGOAL):
            newAction = self.actions.LEFT
        elif self.right(obs, RayCastObjects.GOODGOAL):
            newAction = self.actions.RIGHT
        elif self.ahead(obs, RayCastObjects.BADGOAL):
            newAction = self.actions.BACKWARDS
        elif self.left(obs, RayCastObjects.BADGOAL):
            newAction = self.actions.BACKWARDSLEFT
        elif self.right(obs, RayCastObjects.BADGOAL):
            newAction = self.actions.BACKWARDSRIGHT
        else:
            if self.prev_action == self.actions.NOOP or self.prev_action == self.actions.BACKWARDS:
                newAction = self.actions.LEFT
            elif newAction == self.actions.NOOP:
                pass
            else:
                newAction = self.prev_action    
        self.prev_action = newAction
        return newAction

    def ahead(self, obs, object):
        """Returns true if the input object is ahead of the agent"""
        if(obs[self.listOfObjects.index(object)][int((self.no_rays-1)/2)] > 0):
            # print("found " + str(object) + " ahead")
            return True
        return False

    def left(self, obs, object):
        """Returns true if the input object is left of the agent"""
        for i in range(int((self.no_rays-1)/2)):
            if(obs[self.listOfObjects.index(object)][i] > 0):
                # print("found " + str(object) + " left")
                return True
        return False

    def right(self, obs, object):
        """Returns true if the input object is right of the agent"""
        for i in range(int((self.no_rays-1)/2)):
            if(obs[self.listOfObjects.index(object)][i+int((self.no_rays-1)/2) + 1] > 0):
                # print("found " + str(object) + " right")
                return True
        return False


import random
import numpy as np
from pathlib import Path

from animalai import AnimalAIEnvironment, arenas
from animalai.actions import AAIActions
from animalai.executable import find_executable
from animalai.agents import RandomActionAgent


def main():
    """
    Manual test to see if random action agent works.
    """

    env = AnimalAIEnvironment(
        file_name=str(find_executable(Path("test/executable/"))),
        arenas_configurations=arenas.GOOD_GOAL_RANDOM_POS,
        base_port=5005 + random.randint(0, 1000),
        useRayCasts=True,
        rayMaxDegrees=30,
        raysPerSide=3,
        play=False,
        inference=True,
    )

    randomAgent = RandomActionAgent(step_length_distribution=lambda: np.random.normal(
        5, 1))  # a Rayleigh walker (sampling from normal distribution)

    # by default should be AnimalAI?team=0
    behavior = list(env.behavior_specs.keys())[0]

    # define actions
    actions = AAIActions()

    # Run two episodes
    for _episode in range(2):
        env.step()
        done = False
        episodeReward = 0

        previous_action = actions.NOOP
        new_action = randomAgent.get_new_action(prev_step=previous_action)

        while not done:
            step_list = randomAgent.get_num_steps(prev_step=new_action)

            for action in step_list:
                print(action)
                env.set_actions(behavior, action.action_tuple)
                env.step()
                previous_action = action

                dec, term = env.get_steps(behavior)

                if len(term) > 0:
                    episodeReward += term.reward
                    print(f"Episode Reward: {episodeReward}")
                    done = True
                    break

            new_action = randomAgent.get_new_action(prev_step=previous_action)

    print("Closing environment")
    env.close()
    print("Environment Closed")


if __name__ == "__main__":
    main()



import random
from pathlib import Path

from animalai import AnimalAIEnvironment, arenas
from animalai.executable import find_executable
from animalai.agents import Braitenberg


def main():
    """
    Manual test to see if brainenberg agent works.
    """
    n_rays = 9

    env = AnimalAIEnvironment(
        file_name=str(find_executable(Path("test/executable/"))),
        arenas_configurations=arenas.GOOD_GOAL_RANDOM_POS,
        base_port=5005 + random.randint(0, 1000),
        useRayCasts=True,
        rayMaxDegrees=30,
        raysPerSide=(n_rays - 1) // 2,
        play=False,
        inference=True,
    )

    # A simple BraitenBerg Agent that heads towards food items.
    braitenbergAgent = Braitenberg(no_rays=n_rays)
    # by default should be AnimalAI?team=0
    behavior = list(env.behavior_specs.keys())[0]

    # Run two episodes
    for _episode in range(2):
        episodeReward = 0
        while True:
            # Step the environment
            env.step()
            step, term = env.get_steps(behavior)

            # Episode is over
            if len(term) > 0:
                episodeReward += term.reward
                print(f"Episode Reward: {episodeReward}")
                break  # Go to next episode

            # Get observations
            raycasts = env.get_obs_dict(step.obs)["rays"]
            braitenbergAgent.prettyPrint(raycasts)
            episodeReward += step.reward if len(step.reward) > 0 else 0

            # Get agent action
            action = braitenbergAgent.get_action(raycasts)
            print(action)
            env.set_actions(behavior, action.action_tuple)

    print("Closing environment")
    env.close()
    print("Environment Closed")


if __name__ == "__main__":
    main()



import random

from pathlib import Path

from animalai import AnimalAIEnvironment, arenas
from animalai.executable import find_executable

def play(configuration_file: str = None, env_path: str = None) -> None:
    """
    Load a config file and play
    :param configuration_file: str path to a yaml configuration. Plays animalai.arenas.GoodGoal_Random.yml by default
    :param env_path: str path to AAI environment executable. Looks for it in root by default.
    :return: None
    """

    print("Initializing AAI environment")
    environment = AnimalAIEnvironment(
        file_name=env_path if env_path is not None else str(find_executable(Path(""))),
        base_port=5005 + random.randint(0, 1000),
        arenas_configurations=configuration_file if configuration_file is not None else arenas.GOOD_GOAL_RANDOM_POS,
        play=True,
    )

    print("Press Q in the Unity window then Ctrl+C in the command line to close the environment effectively.")

    # Run the environment until signal is lost
    try:
        while environment._process:  # type: ignore
            continue
    except KeyboardInterrupt:
        pass
    finally:
        environment.close()